from datetime import datetime
from unittest.mock import patch

from django.test import override_settings
from django.urls import reverse

from arkindex.ponos.models import Agent, AgentMode, Farm, State
from arkindex.process.models import Process, ProcessMode
from arkindex.project.tests import FixtureAPITestCase


class TestMetricsAPI(FixtureAPITestCase):

    def test_metrics_base_wrong_port(self):
        response = self.client.get(reverse("metrics:base-metrics"))
        self.assertEqual(response.status_code, 404)

    @override_settings(PROMETHEUS_METRICS_PORT=42, PUBLIC_HOSTNAME="hostname", ARKINDEX_ENV="test")
    def test_metrics_base(self):
        response = self.client.get(reverse("metrics:base-metrics"), SERVER_PORT=42)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'arkindex_instance{hostname="hostname", env="test"} 1')

    @override_settings(PROMETHEUS_METRICS_PORT=42, PUBLIC_HOSTNAME="hostname", ARKINDEX_ENV="test")
    def test_metrics_tasks(self):

        # Retrieve objects
        process = Process.objects.get(mode=ProcessMode.Workers)
        with patch("arkindex.process.tasks.initialize_activity.delay"):
            process.run()
        task1, task2, task3 = process.tasks.all().order_by("depth")

        farm = Farm.objects.first()
        agent = Agent.objects.create(
            mode=AgentMode.Docker,
            hostname="Demo Agent",
            farm=farm,
            last_ping=datetime.now(),
            cpu_cores=42,
            cpu_frequency=42e8,
            ram_total=42e3
        )
        last_ping = int(agent.last_ping.timestamp())

        # Create some tasks with various tasks
        task1.state = State.Pending
        task1.save()
        task2.state = State.Error
        task2.save()
        task3.state = State.Unscheduled
        task3.save()

        response = self.client.get(reverse("metrics:base-metrics"), SERVER_PORT=42)
        self.assertEqual(response.status_code, 200)

        metrics = """arkindex_instance{hostname="hostname", env="test"} 1
arkindex_tasks{hostname="hostname", env="test", state="Pending", farm="Wheat farm"} 1
arkindex_tasks{hostname="hostname", env="test", state="Unscheduled", farm="Wheat farm"} 1
arkindex_agent_ping{hostname="hostname", env="test", agent_name="Demo Agent", farm="Wheat farm"} """ + str(last_ping)
        self.assertEqual(response.content, metrics.encode("utf-8"))
