from django.core.management.base import CommandError

from arkindex.documents.models import Element
from arkindex.project.argparse import CorpusArgument, ElementArgument
from arkindex.project.tests import FixtureTestCase


class TestArgparseHelpers(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.vol = Element.objects.get(name="Volume 1")
        cls.page = Element.objects.get(name="Volume 1, page 1r")

    def test_corpus_argument_id(self):
        arg = CorpusArgument()
        result = arg(str(self.corpus.id))
        self.assertEqual(result, self.corpus)

    def test_corpus_argument_name(self):
        arg = CorpusArgument()
        result = arg("tests")
        self.assertEqual(result, self.corpus)

    def test_corpus_argument_many(self):
        arg = CorpusArgument(many=True)
        result = arg("tests")
        self.assertQuerySetEqual(result, [self.corpus, ])

    def test_corpus_argument_not_found(self):
        arg = CorpusArgument()
        with self.assertRaisesRegex(CommandError, 'Corpus "oh no" does not exist'):
            arg("oh no")

    def test_element_argument_id(self):
        arg = ElementArgument()
        result = arg(str(self.page.id))
        self.assertEqual(result, self.page)

    def test_element_argument_name(self):
        arg = ElementArgument()
        result = arg("volume 1, page 1r")
        self.assertEqual(result, self.page)

    def test_element_argument_not_found(self):
        arg = ElementArgument()
        with self.assertRaisesRegex(CommandError, 'Element "oh no" does not exist'):
            arg("oh no")

    def test_element_argument_filter(self):
        arg = ElementArgument(type__folder=True)
        result = arg("volume 1")
        self.assertEqual(result, self.vol)

    def test_element_argument_filter_not_found(self):
        arg = ElementArgument(type__folder=True)
        with self.assertRaisesRegex(CommandError, 'Element "surface 1" does not exist'):
            arg("surface 1")

    def test_element_argument_many(self):
        arg = ElementArgument(type__folder=True, many=True)
        result = arg("volume 1")
        self.assertQuerySetEqual(result, [self.vol, ])

    def test_element_argument_multiple(self):
        arg = ElementArgument()
        with self.assertRaisesRegex(CommandError, '"volume" matches multiple Element instances'):
            arg("volume")
